@@ -113,9 +113,17 @@ class CLIParser:
                 "-r",
                 type=str,
                 required=False,
-                help="work requirement definition file in JSON format",
+                help="submit a work requirement definition file in JSON format",
                 metavar="<work_requirement.json>",
             )
+            parser.add_argument(
+                "--json-raw",
+                "-j",
+                type=str,
+                required=False,
+                help="submit a 'raw' JSON work requirement file",
+                metavar="<raw_work_requirement.json>",
+            )
             parser.add_argument(
                 "--follow",
                 "-f",
@@ -140,7 +148,10 @@ class CLIParser:
                 metavar="<task_type>",
             )
 
-        if any(module in sys.argv[0] for module in ["provision"] + all_options_modules):
+        if any(
+            module in sys.argv[0]
+            for module in ["provision", "instantiate"] + all_options_modules
+        ):
             parser.add_argument(
                 "--worker-pool",
                 "-p",
@@ -295,11 +306,22 @@ class CLIParser:
         ):
             parser.add_argument(
                 "--dry-run",
+                "-D",
                 action="store_true",
                 required=False,
                 help="dry-run the action and print the JSON that would be submitted",
             )
 
+        if any(module in sys.argv[0] for module in ["instantiate"]):
+            parser.add_argument(
+                "--compute-requirement",
+                "-C",
+                type=str,
+                required=False,
+                help="the compute requirement definition",
+                metavar="<compute_requirement.json>",
+            )
+
         self.args = parser.parse_args()
 
         if self.args.docs:
@@ -444,6 +466,14 @@ class CLIParser:
     def dry_run(self) -> Optional[bool]:
         return self.args.dry_run
 
+    @property
+    def json_raw(self) -> Optional[str]:
+        return self.args.json_raw
+
+    @property
+    def compute_requirement(self) -> Optional[str]:
+        return self.args.compute_requirement
+
 
 def lookup_module_description(module_name: str) -> Optional[str]:
     """
