@@ -2,11 +2,12 @@
 Utilities for applying Mustache substitutions.
 """
 import os
+import re
 import sys
+import tempfile
 from datetime import datetime
 from getpass import getuser
 from io import StringIO
-from json import load as json_load
 from json import loads as json_loads
 from random import randint
 from typing import Dict, List, Optional, Union
@@ -219,22 +220,24 @@ def load_json_file_with_mustache_substitutions(filename: str, prefix: str = "")
     substitutions processed.
     """
     with open(filename, "r") as f:
-        dict_data = json_load(f)
-    process_mustache_substitutions(dict_data, prefix=prefix)
-    return dict_data
+        file_contents = f.read()
+    file_contents = mustache_process_file_contents(file_contents, prefix=prefix)
+    return json_loads(file_contents)
 
 
 def load_jsonnet_file_with_mustache_substitutions(filename: str, prefix="") -> Dict:
     """
-    Takes a JSONNET filename and returns a dictionary with its mustache
+    Takes a Jsonnet filename and returns a dictionary with its mustache
     substitutions processed.
     """
-
     check_jsonnet_import()
     from _jsonnet import evaluate_file
 
-    dict_data = json_loads(evaluate_file(filename))
-    process_mustache_substitutions(dict_data, prefix=prefix)
+    with MustachePreprocessedJsonnetFile(
+        filename=filename, prefix=prefix
+    ) as preprocessed_filename:
+        dict_data = json_loads(evaluate_file(preprocessed_filename))
+
     return dict_data
 
 
@@ -255,3 +258,58 @@ def load_toml_file_with_mustache_substitutions(filename: str, prefix: str = "")
 
     process_mustache_substitutions(config, prefix=prefix)
     return config
+
+
+class MustachePreprocessedJsonnetFile:
+    """
+    The jsonnet 'evaluate_file' function will only operate on files,
+    not strings, so this context manager class will create a
+    temporary, mustache-processed file that can be used by the
+    evaluator, then deleted.
+    """
+
+    def __init__(self, filename: str, prefix: str = ""):
+        self.filename = filename
+        self.prefix = prefix
+
+    def __enter__(self) -> str:
+        """
+        Return the filename of the temporary mustache-processed
+        jsonnet file.
+        """
+        with open(self.filename, "r") as file:
+            file_contents = file.read()
+        processed_file_contents: str = mustache_process_file_contents(
+            file_contents, self.prefix
+        )
+        with tempfile.NamedTemporaryFile(mode="w", delete=False) as temp_file:
+            temp_file.write(processed_file_contents)
+        self.temp_filename: str = temp_file.name
+        return self.temp_filename
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        os.remove(self.temp_filename)
+
+
+def mustache_process_file_contents(file_contents: str, prefix: str) -> str:
+    """
+    Process substitutions in the raw contents of a complete file.
+    """
+    mustache_regex = prefix + "{{[:,A-Z,a-z,0-9,_,-]*}}"
+    m_expressions = set(re.findall(mustache_regex, file_contents))
+    for m_expression in m_expressions:
+        replacement_expression = substitute_mustache_str(m_expression, prefix=prefix)
+        if isinstance(replacement_expression, str):
+            file_contents = file_contents.replace(m_expression, replacement_expression)
+        else:
+            # If the replacement is an int, float, or bool, we need to
+            # remove the enclosing quotes when we substitute, and ensure
+            # that lower case 'false' & 'true' are used. Account for both
+            # double and single quotes (for Jsonnet support).
+            file_contents = file_contents.replace(
+                f'"{m_expression}"', str(replacement_expression).lower()
+            )
+            file_contents = file_contents.replace(
+                f"'{m_expression}'", str(replacement_expression).lower()
+            )
+    return file_contents
