@@ -4,24 +4,33 @@ Functions focused on print outputs.
 
 import sys
 from datetime import datetime
-from typing import List, Optional, TypeVar
+from json import dumps as json_dumps
+from textwrap import indent as text_indent
+from typing import Dict, List, Optional, TypeVar
 
 from tabulate import tabulate
 from yellowdog_client import PlatformClient
+from yellowdog_client.common.json import Json
 from yellowdog_client.model import (
     ComputeRequirementSummary,
+    ComputeRequirementTemplateUsage,
     ConfiguredWorkerPool,
     ObjectPath,
     ProvisionedWorkerPool,
+    ProvisionedWorkerPoolProperties,
     Task,
     TaskGroup,
     WorkerPoolSummary,
+    WorkRequirement,
     WorkRequirementSummary,
 )
 
 from yd_commands.args import ARGS_PARSER
+from yd_commands.compact_json import CompactJSONEncoder
 from yd_commands.object_utilities import Item
 
+JSON_INDENT = 2
+
 
 def print_string(msg: str = "") -> str:
     """
@@ -196,6 +205,14 @@ def print_numbered_object_list(
     print()
 
 
+def print_numbered_strings(objects: List[str]):
+    table = []
+    for index, obj in enumerate(objects):
+        table.append([index + 1, ":", obj])
+    print(indent(tabulate(table, tablefmt="plain"), indent_width=4))
+    print()
+
+
 def sorted_objects(objects: List[Item], reverse: bool = False) -> List[Item]:
     """
     Sort objects by their 'name' property.
@@ -207,4 +224,90 @@ def indent(txt: str, indent_width: int = 4) -> str:
     """
     Indent lines of text.
     """
-    return "\n".join(" " * indent_width + ln for ln in txt.splitlines())
+    return text_indent(txt, prefix=" " * indent_width)
+
+
+def print_json(
+    data: Dict,
+    initial_indent: int = 0,
+    drop_first_line: bool = False,
+    with_final_comma: bool = False,
+):
+    """
+    Print a dictionary as a JSON data structure, using the compact JSON
+    encoder.
+    """
+    json_string = indent(
+        json_dumps(data, indent=JSON_INDENT, cls=CompactJSONEncoder), initial_indent
+    )
+    if drop_first_line:
+        json_string = "\n".join(json_string.splitlines()[1:])
+    if with_final_comma:
+        print(json_string, end=",\n")
+    else:
+        print(json_string)
+
+
+def print_yd_object(
+    yd_object: object,
+    initial_indent: int = 0,
+    drop_first_line: bool = False,
+    with_final_comma: bool = False,
+):
+    """
+    Print a YellowDog object as a JSON data structure,
+    using the compact JSON encoder
+    """
+    print_json(Json.dump(yd_object), initial_indent, drop_first_line, with_final_comma)
+
+
+def print_worker_pool(
+    crtu: ComputeRequirementTemplateUsage, pwpp: ProvisionedWorkerPoolProperties
+):
+    """
+    Reconstruct and print the JSON-formatted Worker Pool specification.
+    """
+    print_log("Dry-run: Printing JSON Worker Pool specification")
+    wp_data = {
+        "provisionedProperties": Json.dump(pwpp),
+        "requirementTemplateUsage": Json.dump(crtu),
+    }
+    print_json(wp_data)
+    print_log("Dry run: Complete")
+
+
+class WorkRequirementSnapshot:
+    """
+    Represent a complete Work Requirement, with Tasks included within
+    Task Group definitions. Note, this is not an 'official' representation
+    of a Work Requirement.
+    """
+
+    def __init__(self):
+        self.wr_data: Dict = {}
+
+    def set_work_requirement(self, wr: WorkRequirement):
+        """
+        Set the Work Requirement to be represented, processed to
+        comply with the API.
+        """
+        self.wr_data = Json.dump(wr)  # Dictionary holding the complete WR
+
+    def add_tasks(self, task_group_name: str, tasks: List[Task]):
+        """
+        Add the list of Tasks to a named Task Group within the
+        Work Requirement. Cumulative.
+        """
+        for task_group in self.wr_data["taskGroups"]:
+            if task_group["name"] == task_group_name:
+                task_group["tasks"] = task_group.get("tasks", [])
+                task_group["tasks"] += [Json.dump(task) for task in tasks]
+                return
+
+    def print(self):
+        """
+        Print the JSON representation
+        """
+        print_log("Dry-run: Printing JSON Work Requirement specification:")
+        print_json(self.wr_data)
+        print_log("Dry-run: Complete")
